<?php
//  +------------------------------------------------------------------------+
//  | netjukebox, Copyright (C) 2001-2017 Willem Bartels                     |
//  |                                                                        |
//  | http://www.netjukebox.nl                                               |
//  | http://forum.netjukebox.nl                                             |
//  |                                                                        |
//  | This program is free software: you can redistribute it and/or modify   |
//  | it under the terms of the GNU General Public License as published by   |
//  | the Free Software Foundation, either version 3 of the License, or      |
//  | (at your option) any later version.                                    |
//  |                                                                        |
//  | This program is distributed in the hope that it will be useful,        |
//  | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
//  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
//  | GNU General Public License for more details.                           |
//  |                                                                        |
//  | You should have received a copy of the GNU General Public License      |
//  | along with this program.  If not, see <http://www.gnu.org/licenses/>.  |
//  +------------------------------------------------------------------------+




//  +------------------------------------------------------------------------+
//  | Execution time                                                         |
//  +------------------------------------------------------------------------+
function executionTime() {
	$request_time_float = isset($_SERVER['REQUEST_TIME_FLOAT']) ? $_SERVER['REQUEST_TIME_FLOAT'] : NJB_REQUEST_TIME_FLOAT;
	$miliseconds = round((microtime(true) - $request_time_float) * 1000);
	$seconds = round($miliseconds / 1000);

	if ($miliseconds < 1000)	return $miliseconds . ' ms';
	if ($seconds == 1)			return '1 second';
	if ($seconds < 60)			return $seconds . ' seconds';
								return formattedTime($miliseconds);
}




//  +------------------------------------------------------------------------+
//  | Formatted time                                                         |
//  +------------------------------------------------------------------------+
function formattedTime($time, $milisecond = true) {
	$seconds 	= ($milisecond) ? round($time / 1000) : $time;
	$hours		= floor($seconds / 3600);
	$minutes 	= floor($seconds / 60) % 60;
	$seconds 	= $seconds % 60;

	if ($hours > 0)	return $hours . ':' . sprintf('%02d:%02d', $minutes, $seconds);
					return $minutes . sprintf(':%02d', $seconds);
}




//  +------------------------------------------------------------------------+
//  | Formatted days                                                         |
//  +------------------------------------------------------------------------+
function formattedDays($seconds) {
	$days = $seconds / 3600 / 24;
	if ($days <= 1)	return  number_format($days, 1, '.', '') . ' day';
					return  number_format($days, 1, '.', '') . ' days';
}




//  +------------------------------------------------------------------------+
//  | Formatted size                                                         |
//  +------------------------------------------------------------------------+
function formattedSize($filesize) {
	$weight = array('bytes', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB');

	for ($i = 0; $filesize >= 1024; $i++)
		$filesize /= 1024;

	if ($i == 0)	return (int) $filesize . ' ' . $weight[$i];
					return number_format($filesize, 1, '.', '') . ' ' . $weight[$i];
}




//  +------------------------------------------------------------------------+
//  | Formatted bitrate                                                      |
//  +------------------------------------------------------------------------+
function formattedBirate($bitrate) {
	$weight = array('bps', 'kbps', 'Mbps', 'Gbps', 'Tbps', 'Pbps', 'Ebps', 'Zbps', 'Ybps');

	for ($i = 0; $bitrate >= 1000; $i++)
		$bitrate /= 1000;

	return round($bitrate) . ' ' . $weight[$i];
}




//  +------------------------------------------------------------------------+
//  | Formatted frequency                                                    |
//  +------------------------------------------------------------------------+
function formattedFrequency($frequency) {
	$weight = array('Hz', 'kHz', 'MHz', 'GHz', 'THz', 'PHz', 'EHz', 'ZHz', 'YHz');

	for ($i = 0; $frequency >= 1000; $i++)
		$frequency /= 1000;

	return number_format($frequency, 1) . ' ' . $weight[$i];
}




//  +------------------------------------------------------------------------+
//  | Formatted date                                                         |
//  +------------------------------------------------------------------------+
function formattedDate($year = null, $month = null, $day = null) {
	$date = '';
	if (isset($day))	$date .= str_pad($day, 2, 0, STR_PAD_LEFT) . '&nbsp;';
	if (isset($month))	$date .= formattedMonth($month) . '&nbsp;';
	if (isset($year))	$date .= $year;

	return $date;
}




//  +------------------------------------------------------------------------+
//  | Formatted month                                                        |
//  +------------------------------------------------------------------------+
function formattedMonth($number) {
	$month = array(1 =>	'January', 'February', 'March', 'April', 'May', 'June',
					'July', 'August', 'September', 'October', 'November', 'December');

	return $month[$number];
}




//  +------------------------------------------------------------------------+
//  | Formatted recent date                                                  |
//  +------------------------------------------------------------------------+
function formattedRecentDate($time) {
	if ($time >= strtotime('today'))					return 'Today';
	if ($time >= strtotime('yesterday'))				return 'Yesterday';
	if ($time >= strtotime('this week'))				return 'This week';
	if ($time >= strtotime('previous week'))			return 'Previous week';
	if ($time >= strtotime('first day of this month'))	return 'This month';
	return date('F Y', $time);
}




//  +------------------------------------------------------------------------+
//  | HTML                                                                   |
//  +------------------------------------------------------------------------+
function html($string) {
	return htmlspecialchars($string, ENT_QUOTES | ENT_HTML5, NJB_DEFAULT_CHARSET);
	// &apos; should be supported with HTML 5
	// https://dev.w3.org/html5/html-author/charref
}




//  +------------------------------------------------------------------------+
//  | Safe JSON encode                                                       |
//  +------------------------------------------------------------------------+
function safe_json_encode($data) {
	if (NJB_DEFAULT_CHARSET == 'UTF-8' && version_compare(PHP_VERSION, '5.4.0', '>='))
		return json_encode($data, JSON_UNESCAPED_UNICODE);
	elseif (NJB_DEFAULT_CHARSET == 'UTF-8')
		return json_encode($data);
	else
		return json_encode(recursive_iconv_to_utf8($data));
}




//  +------------------------------------------------------------------------+
//  | Recursive iconv to utf8                                                |
//  +------------------------------------------------------------------------+
function recursive_iconv_to_utf8($data) {
	if (is_string($data)) return iconv(NJB_DEFAULT_CHARSET, 'UTF-8', $data);
	if (!is_array($data)) return $data;

	$data = array_map('recursive_iconv_to_utf8', $data);

	return $data;
}




//  +------------------------------------------------------------------------+
//  | Escape CMD arg                                                         |
//  +------------------------------------------------------------------------+
function escapeCmdArg($arg) {
	if (NJB_WINDOWS) {
		// No need to escape " because this symbol isn't used by Windows
		return '"' . str_replace('/', '\\', $arg) . '"';
	}
	else {
		// Didn't use escapeshellarg() because of problems with UTF8
		// Thanks Marc Maurice: http://positon.org/php-escapeshellarg-function-utf8-and-locales
		return "'" . str_replace("'", "'\\''", $arg) . "'";
	}
}




//  +------------------------------------------------------------------------+
//  | Encode escape character                                                |
//  +------------------------------------------------------------------------+
function encodeEscapeChar($filename) {
	global $cfg;

	foreach ($cfg['escape_char'] as $key => $value)
		$filename = str_replace($key, $value, $filename); // Example: * to %2A

	// Urlencode to underscore
	foreach ($cfg['escape_char'] as $key => $value) {
		if ($cfg['char_urlencode_to_underscore'] && $value[0] == '%')
			$filename = str_replace($value, '_', $filename);
	}

	return $filename;
}




//  +------------------------------------------------------------------------+
//  | Decode escape character                                                |
//  +------------------------------------------------------------------------+
function decodeEscapeChar($filename) {
	global $cfg;

	foreach ($cfg['escape_char'] as $key => $value)
		$filename = str_replace($value, $key, $filename); // Example: %2A to *

	return $filename;
}




//  +------------------------------------------------------------------------+
//  | Download (zip) filename                                                |
//  +------------------------------------------------------------------------+
function downloadFilename($filename) {
	global $cfg;

	// Decode filename
	$filename = decodeEscapeChar($filename); // Example: %2A to *

	// Encode for client compatibility
	foreach ($cfg['char_client_limit'][NJB_CLIENT_OS] as $key => $value)
		$filename = str_replace($value, $cfg['escape_char'][$value], $filename); // Example: * to %2A

	// Encode for server compatibility
	foreach ($cfg['char_server_limit'][NJB_SERVER_OS] as $key => $value)
		$filename = str_replace($value, $cfg['escape_char'][$value], $filename); // Example: * to %2A

	// Urlencode to underscore
	foreach ($cfg['escape_char'] as $key => $value) {
		if ($cfg['char_urlencode_to_underscore'] && $value[0] == '%')
			$filename = str_replace($value, '_', $filename);
	}

	return $filename;
}




//  +------------------------------------------------------------------------+
//  | UploadFilename                                                         |
//  +------------------------------------------------------------------------+
function uploadFilename($filename) {
	global $cfg;

	// Decode filename
	$filename = decodeEscapeChar($filename); // Example: %2A to *

	// Encode for server compatibility
	foreach ($cfg['char_server_limit'][NJB_SERVER_OS] as $key => $value)
		$filename = str_replace($value, $cfg['escape_char'][$value], $filename); // Example: * to %2A

	return $filename;
}




//  +------------------------------------------------------------------------+
//  | Copy filename                                                          |
//  +------------------------------------------------------------------------+
function copyFilename($filename) {
	global $cfg;

	// Decode filename
	$filename = decodeEscapeChar($filename); // Example: %3F to ?

	// Encode for compatibility
	foreach ($cfg['char_album_copy_limit'] as $key => $value)
		$filename = str_replace($value, $cfg['escape_char'][$value], $filename); // Example: ? to %3F

	return $filename;
}




//  +------------------------------------------------------------------------+
//  | BBcode                                                                 |
//  +------------------------------------------------------------------------+
function bbcode($string) {
	global $cfg;
	$bbcode = array(
		'#\[br\]#s',
		'#\[b\](.*?)\[/b\]#s',
		'#\[i\](.*?)\[/i\]#s',
		'#\[img\]([a-z_]+\.png)\[/img\]#s',
		'#\[url=([a-z]+\.php(?:\?.*)?)\](.*?)\[/url\]#s',
		'#\[url\]((?:http|https|ftp)://.*?)\[/url\]#s',
		'#\[url=((?:http|https|ftp)://.*?)\](.*?)\[/url\]#s',
		'#\[email\]([a-z0-9._%-]+@[a-z0-9.-]+\.[a-z]{2,4})\[/email\]#si');
	$replace = array(
		'<br>',
		'<strong>$1</strong>',
		'<em>$1</em>',
		'<img src="' . $cfg['img'] . '$1" alt="" class="small space">',
		'<a href="$1">$2</a>',
		'<a href="$1">$1</a>',
		'<a href="$1">$2</a>',
		'<a href="mailto:$1">$1</a>');

	$string = html($string);
	$string = preg_replace($bbcode, $replace, $string);
	$string = preg_replace_callback('#\[list\](.*?)\[/list\]#s', 'bblist', $string);

	return $string;
}




//  +------------------------------------------------------------------------+
//  | BBcode list                                                            |
//  +------------------------------------------------------------------------+
function bblist($maches) {
	$list = '';
	$list_array = explode('[*]', $maches[1]);
	foreach ($list_array as $key => $value)	{
		if ($key == 0) $list .= $value;
		else $list .= '<li>' . $value . '</li>';
	}

	return '<ul class="bbcode">' . $list . '</ul>';
}




//  +------------------------------------------------------------------------+
//  | BBcode to txt                                                          |
//  +------------------------------------------------------------------------+
function bbcode2txt($string) {
	$bbcode = array(
		"#\r\n|\n|\r#",
		'#\[br\]#s',
		'#\[b\](.*?)\[/b\]#s',
		'#\[i\](.*?)\[/i\]#s',
		'#\[list\](.*?)\[/list\]#s',
		'#\[\*\]#s',
		'#\[img\]([a-z_]+\.png)\[/img\]#s',
		'#\[url=([a-z]+\.php(?:\?.*)?)\](.*?)\[/url\]#s',
		'#\[url\]((?:http|https|ftp)://.*?)\[/url\]#s',
		'#\[url=((?:http|https|ftp)://.*?)\](.*?)\[/url\]#s',
		'#\[email\]([a-z0-9._%-]+@[a-z0-9.-]+\.[a-z]{2,4})\[/email\]#si');

	$replace = array(
		'',
		"\n",
		'$1',
		'$1',
		'$1',
		"\n* ",
		'',
		'$2 <$1>',
		'<$1>',
		'$2 <$1>',
		'$1');

	return preg_replace($bbcode, $replace, $string);
}




//  +------------------------------------------------------------------------+
//  | BBcode reference title                                                 |
//  +------------------------------------------------------------------------+
function bbcodeReferenceTitle() {
	/*
	$list = '[br]<br>';
	$list .= '[b]bold[/b]<br>';
	$list .= '[i]italic[/i]<br>';
	$list .= '[img]small_back.png[/img]<br>';
	$list .= '[url]http://www.example.com[/url]<br>';
	$list .= '[url=http://www.example.com]example[/url]<br>';
	$list .= '[email]info@example.com[/email]<br>';
	$list .= '[list][*]first[*]second[/list]';
	*/
	$list = 'Basic bbcode support';

	return 'title="' . html($list) . '"';

}




//  +------------------------------------------------------------------------+
//  | Artist link                                                            |
//  +------------------------------------------------------------------------+
function artisLink($artist, $track = true) {
	global $cfg, $db;
	if ($artist == 'Unknown')
		return 'Unknown';
	if (($n = mysqli_num_rows(mysqli_query($db, 'SELECT artist FROM album WHERE artist="' . mysqli_real_escape_string($db, $artist) . '"'))) && $n >= 1)
		return '<a href="index.php?action=view2&amp;artist=' . rawurlencode($artist) . '&amp;order=year" title="' . $n . (($n == 1) ? ' Album' : ' Albums') . '">' . html($artist) . '</a>';
	elseif ($track && $n = mysqli_num_rows(mysqli_query($db, 'SELECT artist FROM track WHERE artist="' . mysqli_real_escape_string($db, $artist) . '"')))
		return '<a href="index.php?action=view3all&amp;artist=' . rawurlencode($artist) . '&amp;order=title" title="' . $n . (($n == 1) ? ' Track' : ' Tracks') . '">' . html($artist) . '</a>';
	else
		return html($artist);
}




//  +------------------------------------------------------------------------+
//  | Image title                                                            |
//  +------------------------------------------------------------------------+
function imageTitle($image_id) {
	return 'title="[img]image.php?image_id=' . rawurlencode($image_id) . '[/img]"';
}




//  +------------------------------------------------------------------------+
//  | Access info title                                                      |
//  +------------------------------------------------------------------------+
function accessInfoTitle($access) {
	switch ($access) {
		case 'media':		$info = 'View media';				break;
		case 'popular':		$info = 'View popular albums';		break;
		case 'favorite':	$info = 'View favorites';			break;
		case 'cover':		$info = 'Inline pdf cover';			break;
		case 'stream':		$info = 'Stream media';				break;
		case 'download':	$info = 'Download media';			break;
		case 'upload':		$info = 'Upload media';				break;
		case 'play':		$info = 'Play media';				break;
		case 'add':			$info = 'Add media to playlist';	break;
		case 'statistics':	$info = 'View media statistics';	break;
		case 'admin':		$info = 'Administrator';			break;
	}

	return 'title="' . html($info) .'"';
}




//  +------------------------------------------------------------------------+
//  | Navigator Player Profile                                               |
//  +------------------------------------------------------------------------+
function navPlayerProfile() {
	global $cfg, $db, $nav;
	require_once(NJB_HOME_DIR . 'include/play.inc.php');

	$query = mysqli_query($db, 'SELECT player_name, player_id FROM player ORDER BY player_name');
	if (mysqli_num_rows($query) == 1)
		return;

	if (@$_GET['navigator'] == 'selectPlayerProfile') {
		$nav = array();
		if ($cfg['menu'] == 'index') {
			$nav['name'][]	= 'Media';
			$nav['url'][]	= 'index.php';
			$nav['class'][]	= 'nav';
		}
		elseif ($cfg['menu'] == 'favorite') {
			$nav['name'][]	= 'Favorites';
			$nav['url'][]	= 'favorite.php';
			$nav['class'][]	= 'nav';
		}
		elseif ($cfg['menu'] == 'playlist') {
			$nav['name'][]	= 'Playlist';
			$nav['url'][]	= 'playlist.php';
			$nav['class'][]	= 'nav';
		}

		$nav['name'][]	= 'Player profile';
		$nav['url'][]	= null;
		$nav['class'][]	= 'nav';

		while ($player = mysqli_fetch_assoc($query)) {
			$nav['name'][]	= $player['player_name'];
		 	$nav['url'][]	= 'config.php?action=setPlayerProfile&amp;player_id=' . $player['player_id'] . '&amp;sign=' . $cfg['sign'] . '&amp;menu=' . $cfg['menu'];
		 	$nav['class'][]	= 'suggest';
		}
	}
	else {
		$nav['name'][]	= $cfg['player_name'];
		$nav['url'][]	= $cfg['menu'] . '.php?navigator=selectPlayerProfile';
		$nav['class'][]	= 'nav';
	}
}




//  +------------------------------------------------------------------------+
//  | Random sid                                                             |
//  +------------------------------------------------------------------------+
function randomSid() {
	$uniqid	= uniqid();
	$rand = randomHex(60 - strlen($uniqid));
	return strrev(hex_to_base64url($uniqid . $rand));
}




//  +------------------------------------------------------------------------+
//  | Random file sid                                                        |
//  +------------------------------------------------------------------------+
function randomFileSid() {
	$uniqid	= uniqid();
	$rand = randomHex(40 - strlen($uniqid));
	return strrev($uniqid . $rand);
}




//  +------------------------------------------------------------------------+
//  | Random base64url                                                       |
//  +------------------------------------------------------------------------+
function randomBase64url($lenght = 40) {
	$bytes = ceil($lenght * 0.75);
	$rand = base64url_encode(randomBytes($bytes));
	return substr($rand, 0, $lenght);
}




//  +------------------------------------------------------------------------+
//  | Random hex                                                             |
//  +------------------------------------------------------------------------+
function randomHex($lenght) {
	$bytes = ceil($lenght / 2);
	$rand = bin2hex(randomBytes($bytes));
	return substr($rand, 0, $lenght);
}




//  +------------------------------------------------------------------------+
//  | Random bytes                                                           |
//  +------------------------------------------------------------------------+
function randomBytes($bytes) {
	if  (function_exists('random_bytes') && ($rand = @random_bytes($bytes))); // PHP >= 7
	elseif (is_readable('/dev/arandom') && ($rand = @file_get_contents('/dev/arandom', null, null, null, $bytes)));
	elseif (is_readable('/dev/urandom') && ($rand = @file_get_contents('/dev/urandom', null, null, null, $bytes)));
	elseif (function_exists('openssl_random_pseudo_bytes') && ($rand = @openssl_random_pseudo_bytes($bytes, $crypto_strong)) && $crypto_strong); // PHP >= 5.3.0
	else message(__FILE__, __LINE__, 'error', '[b]Failed to generate cryptographically secure pseudo-random bytes.[/b]');
	return $rand;
}




//  +------------------------------------------------------------------------+
//  | HEX to Base64url                                                       |
//  +------------------------------------------------------------------------+
function hex_to_base64url($hex) {
	return base64url_encode(pack('H*', $hex));
}




//  +------------------------------------------------------------------------+
//  | base64url encode                                                       |
//  +------------------------------------------------------------------------+
function base64url_encode($data) {
	// http://www.ietf.org/rfc/rfc4648.txt
	return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
}




//  +------------------------------------------------------------------------+
//  | base64url decode                                                       |
//  +------------------------------------------------------------------------+
function base64url_decode($data) {
	// Thanks gutzmer at usa dot net
	// http://php.net/manual/en/function.base64-encode.php#103849
	return base64_decode(str_pad(strtr($data, '-_', '+/'), strlen($data) % 4, '=', STR_PAD_RIGHT));
}




//  +------------------------------------------------------------------------+
//  | Filemtime compare                                                      |
//  +------------------------------------------------------------------------+
function filemtimeCompare($filemtime1, $filemtime2) {
	if ($filemtime1 == $filemtime2) return true;
	if (NJB_WINDOWS && $filemtime1 == $filemtime2 + 3600) return true;
	if (NJB_WINDOWS && $filemtime1 == $filemtime2 - 3600) return true;

	return false;
}




//  +------------------------------------------------------------------------+
//  | Source file                                                            |
//  +------------------------------------------------------------------------+
function sourceFile($extension, $bitrate, $id) {
	global $cfg;
	if ($id == -1 ||
		array_key_exists($extension, $cfg['decode_stdout']) == false ||
		$extension == $cfg['encode_extension'][$id] &&
		$bitrate <= round($cfg['encode_bitrate'][$id] * $cfg['transcode_treshold'] / 100))
		return true;
	else
		return false;
}




//  +------------------------------------------------------------------------+
//  | Validate skin                                                          |
//  +------------------------------------------------------------------------+
function validateSkin($skin) {
	$dir = NJB_HOME_DIR . 'skin/' . $skin . '/';

	if (file_exists($dir . 'style.css') &&
		file_exists($dir . 'template.footer.php') &&
		file_exists($dir . 'template.header.php') &&
		preg_match('#^[a-zA-Z0-9-_]+$#', $skin)
		)
		return true;
	else
		return false;
}




//  +------------------------------------------------------------------------+
//  | Update counter: play / add / stream / download / cover                 |
//  +------------------------------------------------------------------------+
function updateCounter($album_id, $flag){
	global $cfg, $db;
	// flag 0 = play/add
	// flag 1 = stream
	// flag 2 = download
	// flag 3 = cover

	$query = mysqli_query($db, 'SELECT time FROM counter
		WHERE album_id	= "' . mysqli_real_escape_string($db, $album_id) . '"
		AND sid			= BINARY "' . mysqli_real_escape_string($db, $cfg['sid']) . '"
		AND flag		= ' . (int) $flag . '
		ORDER BY time DESC LIMIT 1');
	$counter = mysqli_fetch_assoc($query);
	$counter_time = $counter['time'];

	if ($counter_time + 60 - time() < 0) {
		mysqli_query($db, 'INSERT INTO counter (sid, album_id, user_id, flag, time) VALUES (
			"' . mysqli_real_escape_string($db, $cfg['sid']) . '",
			"' . mysqli_real_escape_string($db, $album_id) . '",
			' . (int) $cfg['user_id'] . ',
			' . (int) $flag . ',
			' . (int) time() . ')');
	}
	else {
		mysqli_query($db, 'UPDATE counter
			SET time = 			' . (int) time() . '
			WHERE album_id = 	"' . mysqli_real_escape_string($db, $album_id) . '"
			AND sid =			BINARY "' . mysqli_real_escape_string($db, $cfg['sid']) . '"
			AND flag =			' . (int) $flag . ',
			AND time =			' . (int) $counter_time);
	}
}




//  +------------------------------------------------------------------------+
//  | Create hiden dir                                                       |
//  +------------------------------------------------------------------------+
function createHiddenDir($dir) {
	$file = $dir . 'index.php';
	$content = '<!doctype html><html><head><title></title></head><body></body></html>';

	if (is_dir($dir) == false && @mkdir($dir, 0777) == false)
		message(__FILE__, __LINE__, 'error', '[b]Failed to create directory:[/b][br]' . $dir);

	if (@filesize($file) != strlen($content) && file_put_contents($file, $content) === false)
		message(__FILE__, __LINE__, 'error', '[b]Failed to create file:[/b][br]' . $file);
}




//  +------------------------------------------------------------------------+
//  | Recursive rmdir                                                        |
//  +------------------------------------------------------------------------+
function rrmdir($dir) {
	$entries = @scandir($dir) or message(__FILE__, __LINE__, 'error', '[b]Failed to open directory:[/b][br]' . $dir);
	foreach ($entries as $entry) {
		if ($entry != '.' && $entry != '..') {
			if (is_dir($dir . $entry . '/'))	rrmdir($dir . $entry . '/');
			else								@unlink($dir . $entry) or message(__FILE__, __LINE__, 'error', '[b]Failed to delete file:[/b][br]' . $dir . $entry);
		}
	}
	@rmdir($dir) or message(__FILE__, __LINE__, 'error', '[b]Failed to delete directory:[/b][br]' . $dir);
}




//  +------------------------------------------------------------------------+
//  | Recursive filesize                                                     |
//  +------------------------------------------------------------------------+
function recursiveFilesize($dir, &$size = 0) {
    if (is_readable($dir) === false) return 0;
	$entries = @scandir($dir);
	foreach ($entries as $entry) {
		if ($entry != '.' && $entry != '..') {
			if (is_dir($dir . $entry . '/'))	recursiveFilesize($dir . $entry . '/', $size);
			elseif (is_file($dir . $entry))		$size += @filesize($dir . $entry);
		}
	}
	return $size;
}




//  +------------------------------------------------------------------------+
//  | Resample image                                                         |
//  +------------------------------------------------------------------------+
function resampleImage($image, $size = NJB_IMAGE_SIZE) {
	$extension = strtolower(substr(strrchr($image, '.'), 1));

	if		($extension == 'jpg')	$src_image = @imageCreateFromJpeg($image)	or message(__FILE__, __LINE__, 'error', '[b]Failed to resample image:[/b][br]' . $image);
	elseif	($extension == 'png')	$src_image = @imageCreateFromPng($image)	or message(__FILE__, __LINE__, 'error', '[b]Failed to resample image:[/b][br]' . $image);
	else																		message(__FILE__, __LINE__, 'error', '[b]Failed to resample image:[/b][br]Unsupported extension.');

	if ($extension == 'jpg' && imageSX($src_image) == $size && imageSY($src_image) == $size) {
		$data = @file_get_contents($image) or message(__FILE__, __LINE__, 'error', '[b]Failed to open file:[/b][br]' . $image);
	}
	elseif (imageSY($src_image) / imageSX($src_image) <= 1) {
		// Crops from left and right to get a squire image.
		$sourceWidth		= imageSY($src_image);
		$sourceHeight		= imageSY($src_image);
		$sourceX			= round((imageSX($src_image) - imageSY($src_image)) / 2);
		$sourceY			= 0;
	}
	else {
		// Crops from top and bottom to get a squire image.
		$sourceWidth		= imageSX($src_image);
		$sourceHeight		= imageSX($src_image);
		$sourceX			= 0;
		$sourceY			= round((imageSY($src_image) - imageSX($src_image)) / 2);
	}
	if (isset($sourceWidth)) {
		$dst_image = ImageCreateTrueColor($size, $size);
		imageCopyResampled($dst_image, $src_image, 0, 0, $sourceX, $sourceY, $size, $size, $sourceWidth, $sourceHeight);
		ob_start();
		imageJpeg($dst_image, NULL, NJB_IMAGE_QUALITY);
		$data = ob_get_contents();
		ob_end_clean();
		imageDestroy($dst_image);
	}

	imageDestroy($src_image);
	return $data;
}
