//  +------------------------------------------------------------------------+
//  | netjukebox, Copyright (C) 2001-2017 Willem Bartels                     |
//  |                                                                        |
//  | http://www.netjukebox.nl                                               |
//  | http://forum.netjukebox.nl                                             |
//  |                                                                        |
//  | This program is free software: you can redistribute it and/or modify   |
//  | it under the terms of the GNU General Public License as published by   |
//  | the Free Software Foundation, either version 3 of the License, or      |
//  | (at your option) any later version.                                    |
//  |                                                                        |
//  | This program is distributed in the hope that it will be useful,        |
//  | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
//  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
//  | GNU General Public License for more details.                           |
//  |                                                                        |
//  | You should have received a copy of the GNU General Public License      |
//  | along with this program.  If not, see <http://www.gnu.org/licenses/>.  |
//  +------------------------------------------------------------------------+




//  +------------------------------------------------------------------------+
//  | Initialize                                                             |
//  +------------------------------------------------------------------------+
function init(debug) {
	if (typeof XMLHttpRequest == 'undefined' || typeof JSON.parse != 'function' || typeof Storage == 'undefined' || typeof document.getElementById('menu').classList != 'object') {
		var info = 'Your browser is not supported anymore.\nPlease update to a more recent one.';
		if (debug) {
			info += '\n\nDebug info:\n\n';
			info += 'typeof XMLHttpRequest: ' + typeof XMLHttpRequest + '\n';
			info += 'typeof JSON.parse: ' + typeof JSON.parse + '\n';
			info += 'typeof Storage: ' + typeof Storage + '\n';
			info += 'typeof classList: ' + typeof document.getElementById('menu').classList + '\n';
			info += 'navigator.UserAgent: ' + navigator.userAgent;
		}
		alert(info);
	}
	else {
		// HTML 5 audio support detection
		if (document.cookie.indexOf('netjukebox_canplay_lossy=') == -1) {
			var canplay_lossy = [];
			var canplay_lossless = [];
			var audio = document.createElement('audio');
			if (typeof audio.canPlayType == 'function' && !navigator.userAgent.match(/iPad|iPod|iPhone/)) {
				if (!!audio.canPlayType('audio/mp4; codecs="mp4a.40.2"'))	canplay_lossy.push('m4a');
				if (!!audio.canPlayType('audio/mpeg; codecs="mp3"'))		canplay_lossy.push('mp3');
				if (!!audio.canPlayType('audio/ogg; codecs="vorbis"'))		canplay_lossy.push('ogg');
				if (!!audio.canPlayType('audio/flac'))						canplay_lossless.push('flac');
			}
			document.cookie = 'netjukebox_canplay_lossy=' + canplay_lossy.join();
			document.cookie = 'netjukebox_canplay_lossless=' + canplay_lossless.join();
		}
		tooltip.init();
		if (typeof initialize == 'function')
			initialize();
	}
}




//  +------------------------------------------------------------------------+
//  | Formatted time                                                         |
//  +------------------------------------------------------------------------+
function formattedTime(miliseconds) {
	var seconds 	= Math.round(miliseconds / 1000);
	var hour 		= Math.floor(seconds / 3600);
	var minutes 	= Math.floor(seconds / 60) % 60;
	seconds 		= seconds % 60;

	if (hour > 0)	return hour + ':' + zeroPad(minutes, 2) + ':' + zeroPad(seconds, 2);
	else			return minutes + ':' + zeroPad(seconds, 2);
}




//  +------------------------------------------------------------------------+
//  | Zero pad                                                               |
//  +------------------------------------------------------------------------+
function zeroPad(number, n) {
	var zeroPad = '' + number;

	while(zeroPad.length < n)
		zeroPad = '0' + zeroPad;

	return zeroPad;
}




//  +------------------------------------------------------------------------+
//  | Show hide                                                              |
//  +------------------------------------------------------------------------+
function showHide(a, b) {
	document.getElementById(a).style.display = (document.getElementById(a).style.display == 'none') ? 'block' : 'none';
	document.getElementById(b).style.display = (document.getElementById(b).style.display == 'none') ? 'block' : 'none';
}




//  +------------------------------------------------------------------------+
//  | Inverse checkbox                                                       |
//  +------------------------------------------------------------------------+
function inverseCheckbox(frm) {
	for (var i = 0; i < frm.elements.length; i++) {
		if (frm.elements[i].type == 'checkbox')
			frm.elements[i].checked = !frm.elements[i].checked;
	}
}




//  +------------------------------------------------------------------------+
//  | Get relative X                                                         |
//  +------------------------------------------------------------------------+
function getRelativeX(e, o) {
	if (e.offsetX) {
		return e.offsetX;
	}
	if (e.pageX && o.offsetParent) {
		var x = e.pageX;
		while (o) {
			x -= o.offsetLeft;
			o = o.offsetParent;
		}
		return x;
	}
	return 0;
}




//  +---------------------------------------------------------------------------+
//  | AJAX request                                                              |
//  +---------------------------------------------------------------------------+
function ajaxRequest(url, callback, postData) {
	if (typeof timer == 'function')
		timer();
	url += (url.indexOf('?') == -1) ? '?ajax=1' : '&ajax=1';
	var method = (postData) ? 'post' : 'get';
	var http = new XMLHttpRequest();
	http.open(method, url, true);
	if (postData)
	    http.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
	http.onreadystatechange = function() {
		if (http.readyState == 4 && http.status == 200 && typeof callback == 'function') {
			var data = JSON.parse(http.responseText);
			callback(data);
		}
		else if (http.readyState == 4 && http.status == 500) {
			var url = JSON.parse(http.responseText);
			if (url.substr(0,7) == 'http://' || url.substr(0,8) == 'https://')
				location.href = url;
		}
	}
	http.send(postData);
}
