//  +------------------------------------------------------------------------+
//  | netjukebox, Copyright (C) 2001-2017 Willem Bartels                     |
//  |                                                                        |
//  | http://www.netjukebox.nl                                               |
//  | http://forum.netjukebox.nl                                             |
//  |                                                                        |
//  | This program is free software: you can redistribute it and/or modify   |
//  | it under the terms of the GNU General Public License as published by   |
//  | the Free Software Foundation, either version 3 of the License, or      |
//  | (at your option) any later version.                                    |
//  |                                                                        |
//  | This program is distributed in the hope that it will be useful,        |
//  | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
//  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
//  | GNU General Public License for more details.                           |
//  |                                                                        |
//  | You should have received a copy of the GNU General Public License      |
//  | along with this program.  If not, see <http://www.gnu.org/licenses/>.  |
//  +------------------------------------------------------------------------+
//  | This script is based on:                                               |
//  |                                                                        |
//  | Coded by Travis Beckham                                                |
//  | http://www.squidfingers.com                                            |
//  | http://www.podlob.com                                                  |
//  | If want to use this code, feel free to do so,                          |
//  | but please leave this message intact.                                  |
//  |                                                                        |
//  | Internet archive:                                                      |
//  | http://web.archive.org/web/20050823074019/http://www.squidfingers.com/ |
//  +------------------------------------------------------------------------+
var tooltip = {
	id			: 'tooltip',
	tags		: ['a', 'tr', 'td', 'div', 'input'],
	disable		: /android|blackberry|ipad|ipod|mobi|palmos|phone|symbian|webos/i,
	dx			: 15,
	dy			: 15,
	tip			: null
}




tooltip.init = function () {
	if (navigator.userAgent.match(this.disable)) return;
	this.tip = document.getElementById(this.id);
	if (this.tip) document.onmousemove = function(e) {tooltip.move(e)}
	var elements, element, title;
	for (var i = 0; i < this.tags.length; i++) {
		elements = document.getElementsByTagName(this.tags[i]);
		for (var j = 0; j < elements.length; j++) {
			element = elements[j];
			title = element.getAttribute('title');
			if (title) {
				element.setAttribute('tooltip', title);
				element.removeAttribute('title');
				element.onmouseover = function() {tooltip.show(this.getAttribute('tooltip'))}
				element.onmouseout = function() {tooltip.hide()}
			}
		}
	}
}




tooltip.move = function(e) {
	this.tip.style.left = (e.pageX + this.dx) + 'px';
	this.tip.style.top = (e.pageY + this.dy) + 'px';
}




tooltip.show = function(text) {
	if (!this.tip) return;
	this.tip.innerHTML = tooltip.bbcode(text);
	this.tip.style.display = 'block';
}




tooltip.hide = function() {
	if (!this.tip) return;
	this.tip.style.display = 'none';
	this.tip.innerHTML = '';
}




tooltip.bbcode = function(text) {
	return text
		.replace(/&/g, '&amp;')
		.replace(/</g, '&lt;')
		.replace(/>/g, '&gt;')
		.replace(/"/g, '&quot;')
		.replace(/'/g, '&apos;')
		.replace(/\[br\]/g, '<br>')
		.replace(/\[hr\]/g, '<hr>')
		.replace(/\[b\](.+?)\[\/b\]/g, '<strong>$1</strong>')
		.replace(/\[img\](.+?tiny_[a-z]+?\.png)\[\/img\]/g, '<img src="$1" alt="" class="tiny">')
		.replace(/\[img\](image\.php\?image_id=[a-z0-9_]+?)\[\/img\]/g, '<img src="$1" alt="" class="tooltip">')
		.replace(/\[bar\]([0-9]+?%)\[\/bar\]/g, '<div class="bar"><div style="width: $1;"></div></div>');
}
