<?php
//  +------------------------------------------------------------------------+
//  | netjukebox, Copyright (C) 2001-2017 Willem Bartels                     |
//  |                                                                        |
//  | http://www.netjukebox.nl                                               |
//  | http://forum.netjukebox.nl                                             |
//  |                                                                        |
//  | This program is free software: you can redistribute it and/or modify   |
//  | it under the terms of the GNU General Public License as published by   |
//  | the Free Software Foundation, either version 3 of the License, or      |
//  | (at your option) any later version.                                    |
//  |                                                                        |
//  | This program is distributed in the hope that it will be useful,        |
//  | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
//  | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
//  | GNU General Public License for more details.                           |
//  |                                                                        |
//  | You should have received a copy of the GNU General Public License      |
//  | along with this program.  If not, see <http://www.gnu.org/licenses/>.  |
//  +------------------------------------------------------------------------+




//  +------------------------------------------------------------------------+
//  | pdf.php                                                                |
//  +------------------------------------------------------------------------+
require_once('include/initialize.inc.php');
require_once('include/stream.inc.php');
require_once('lib/tcpdf/tcpdf.php');

$action		= @$_GET['action'];
$album_id	= @$_GET['album_id'];

if	($action != 'downloadPDF')
	message(__FILE__, __LINE__, 'error', '[b]Unsupported input value for[/b][br]action');


$query = mysqli_query($db, 'SELECT artist_alphabetic, album, year FROM album WHERE album_id = "' . mysqli_real_escape_string($db, $album_id) . '"');
$album = mysqli_fetch_assoc($query);

if ($album == false)
	message(__FILE__, __LINE__, 'error', '[b]Error[/b][br]album_id not found in database');

authenticateStream(true);

$query = mysqli_query($db, 'SELECT image_front, image_back, image_front_width * image_front_height AS front_resolution, album_id
	FROM bitmap
	WHERE album_id = "' . mysqli_real_escape_string($db, $album_id) . '"');
$bitmap	= mysqli_fetch_assoc($query);





//  +------------------------------------------------------------------------+
//  | Page setup                                                             |
//  +------------------------------------------------------------------------+
$paperWidth		= isset($cfg['pdf_page_width']) ? $cfg['pdf_page_width'] : 210;
$paperHeight	= isset($cfg['pdf_page_height']) ? $cfg['pdf_page_height'] : 297;

// Front cover
$frontWidth		= 121;
$frontHeight	= 120;

// Back cover
$backWidth		= 151;
$backHeight		= 118;
$backFolding	= 6;
$backTextMargin = 8;

// Upper left front cover
$x1 = round(($paperWidth - $frontWidth) / 2);
$y1 = round(($paperHeight - $frontHeight - $backHeight) / 3);

// Upper left back cover
$x2 = round(($paperWidth - $backWidth) / 2);
$y2 = $paperHeight - $y1 - $backHeight;




//  +------------------------------------------------------------------------+
//  | Initialize                                                             |
//  +------------------------------------------------------------------------+
$hash_data = NJB_VERSION . $paperWidth . $paperHeight;
$pdf = new TCPDF('P', 'mm', array($paperWidth, $paperHeight));
$pdf->SetCreator('netjukebox ' . NJB_VERSION);
$pdf->SetPrintHeader(false);
$pdf->SetAutoPageBreak(false);
$font = TCPDF_FONTS::addTTFfont('lib/fonts/ttf/Roboto-Regular.ttf', 'TrueTypeUnicode', '', 32);
$pdf->SetFont($font);
$pdf->AddPage();




//  +------------------------------------------------------------------------+
//  | Header                                                                 |
//  +------------------------------------------------------------------------+
$pdf->SetFontSize(8);
$pdf->SetXY(0, round($y1 / 2));
$pdf->Cell($paperWidth, 0, 'Created with netjukebox ' . NJB_VERSION . ' using TCPDF', 0 , 0, 'C');




//  +------------------------------------------------------------------------+
//  | Front cover                                                            |
//  +------------------------------------------------------------------------+
cuttingLines($x1, $y1, $frontWidth, $frontHeight);

if ($bitmap['front_resolution'] >= $cfg['image_front_treshold']) {
	$pdf->Image($cfg['media_dir'] . $bitmap['image_front'], $x1, $y1, $frontWidth, $frontHeight);
	$hash_data .= '-' . filesize($cfg['media_dir'] . $bitmap['image_front']) . '-' . filemtime($cfg['media_dir'] . $bitmap['image_front']);
}




//  +------------------------------------------------------------------------+
//  | Back cover                                                             |
//  +------------------------------------------------------------------------+
cuttingLines($x2, $y2, $backWidth, $backHeight);
foldingLines($x2, $y2, $backWidth, $backHeight, array($backFolding, $backWidth - $backFolding));

if ($bitmap['image_back']) {
	$pdf->Image($cfg['media_dir'] . $bitmap['image_back'] ,$x2 , $y2, $backWidth, $backHeight);
	$hash_data .= '-' . filesize($cfg['media_dir'] . $bitmap['image_back']) . '-' . filemtime($cfg['media_dir'] . $bitmap['image_back']);
}
else {
	$single_artist = false;
	$query = mysqli_query($db, 'SELECT artist FROM track WHERE album_id = "' . mysqli_real_escape_string($db, $album_id) . '" GROUP BY artist');
	if (mysqli_num_rows($query) == 1)
		$single_artist = true;

	$text = '';
	$previous_disc = 1;
	$query = mysqli_query($db, 'SELECT title, artist, disc FROM track WHERE album_id = "' . mysqli_real_escape_string($db, $album_id) . '" ORDER BY relative_file');
	while ($track = mysqli_fetch_assoc($query)) {
		if ($previous_disc != $track['disc'])	$text .= "\n";
		if ($single_artist) 					$text .= $track['title'] . "\n";
		else 									$text .= $track['artist'] . ' - ' . $track['title'] . "\n";
		$previous_disc = $track['disc'];
	}
	// Back cover text
	$text = iconv(NJB_DEFAULT_CHARSET, 'UTF-8', $text);
	$pdf->SetFontSize(10);
	$pdf->SetXY($x2 + $backFolding + $backTextMargin, $y2 + $backTextMargin);
	$pdf->MultiCell($backWidth - 2 * $backFolding - 2 * $backTextMargin, 0, $text, 0, 'C', null, null, null, null, null, null, null, null, $backHeight - 2 * $backTextMargin);

	// Title
	if (in_array(strtolower($album['artist_alphabetic']), $cfg['no_album_artist']))		$title = $album['album'];
	else 																				$title = $album['artist_alphabetic'] . ' - ' . $album['album'];
	$title = iconv(NJB_DEFAULT_CHARSET, 'UTF-8', $title);

	$pdf->SetFontSize(13);
	// Left title
	$pdf->StartTransform();
	$pdf->SetXY($x2, $y2 + $backHeight);
	$pdf->Rotate(90);
	$pdf->Cell($backHeight, $backFolding, $title);
	$pdf->StopTransform();
	// Right title
	$pdf->StartTransform();
	$pdf->SetXY($x2 + $backWidth, $y2);
	$pdf->Rotate(-90);
	$pdf->Cell($backHeight, $backFolding, $title);
	$pdf->StopTransform();

	$hash_data .= '-' . $text . '-' . $title;
}




//  +------------------------------------------------------------------------+
//  | Output inline PDF                                                      |
//  +------------------------------------------------------------------------+
$filename	= $album['artist_alphabetic'] . ' - ';
$filename	.= ($album['year']) ? $album['year'] . ' - ' : '';
$filename	.= $album['album'] . '.pdf';
$filename	= encodeEscapeChar($filename);

$data = $pdf->Output(null, 'S');
$etag = '"' . md5($hash_data) . '"';
streamData($data, 'application/pdf', 'inline', $filename, $etag);

updateCounter($album_id, NJB_COUNTER_COVER);
exit();




//  +------------------------------------------------------------------------+
//  | Cutting Lines                                                          |
//  +------------------------------------------------------------------------+
function cuttingLines($x, $y, $width, $height, $spacer = 2, $line = 8) {
	global $pdf;
	$sum = $spacer + $line;
	$pdf->SetLineWidth(0.2);

	// Upper left
	$x = $x;
	$y = $y;
	$pdf->Line($x - $sum, $y, $x - $spacer, $y);
	$pdf->Line($x, $y - $sum, $x, $y - $spacer);

	// Upper right
	$x = $x + $width;
	$y = $y;
	$pdf->Line($x, $y - $sum, $x, $y - $spacer);
	$pdf->Line($x + $sum, $y, $x + $spacer, $y);

	// Lower right
	$x = $x;
	$y = $y + $height;
	$pdf->Line($x + $sum, $y, $x + $spacer, $y);
	$pdf->Line($x, $y + $sum, $x, $y + $spacer);

	// Lower left
	$x = $x - $width;
	$y = $y;
	$pdf->Line($x, $y + $sum, $x, $y + $spacer);
	$pdf->Line($x - $sum, $y, $x - $spacer, $y);
}




//  +------------------------------------------------------------------------+
//  | Folding Lines                                                          |
//  +------------------------------------------------------------------------+
function foldingLines($x, $y, $width, $height, $tabs, $spacer = 2, $line = 8) {
	global $pdf;
	$sum = $spacer + $line;
	$pdf->SetLineWidth(0.2);

	foreach ($tabs as $key => $tab) {
		$tab = $tab + $x;
		$pdf->Line($tab, $y - $sum, $tab, $y - $spacer);
		$pdf->Line($tab, $y + $height + $sum, $tab, $y + $height + $spacer);
	}
}
